import re
import random
import hashlib
from django.db import models

SHA1_RE = re.compile('^[a-f0-9]{40}$')

class RegistrationManager(models.Manager):
    """Custom manager for the ``RegistrationProfile`` model.

    The methods defined here provide shortcuts for account creation
    and activation (including generation and emailing of activation
    keys), and for cleaning out expired inactive accounts.
    """
    def activate_user(self, activation_key):
        """Validate an activation key and activate the corresponding
        ``User`` if valid.

        If the key is valid and has not expired, return the ``User``
        after activating.

        If the key is not valid or has expired, return ``False``.

        If the key is valid but the ``User`` is already active,
        return ``False``.
        """
        # Make sure the key we're trying conforms to the pattern of a
        # SHA1 hash; if it doesn't, no point trying to look it up in
        # the database.
        if SHA1_RE.search(activation_key):
            try:
                profile = self.get(activation_key=activation_key)
            except self.model.DoesNotExist:
                return

            return profile.activate()

    def create_profile(self, user):
        """Create a ``RegistrationProfile`` for a given ``User``, and return
        the ``RegistrationProfile``.

        The activation key for the ``RegistrationProfile`` will be a SHA1 hash,
        generated from a combination of the ``User``'s username and a random
        salt.
        """
        salt = hashlib.sha1(str(random.random())).hexdigest()[:5]
        username = user.username

        if isinstance(username, unicode):
            username = username.encode('utf-8')

        activation_key = hashlib.sha1(salt+username).hexdigest()
        return self.create(user=user, activation_key=activation_key)



from django.dispatch import Signal

# A new user has registered.
user_registered = Signal(providing_args=['user', 'request', 'backend'])

# A new user has verified their registration.
user_verified = Signal(providing_args=['user', 'request', 'backend'])

# A user has activated his or her account.
user_activated = Signal(providing_args=['user', 'request', 'backend'])

# A user has been moderated.
user_moderated = Signal(providing_args=['user', 'request', 'backend'])



import re
import random
from django import forms
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.conf.urls import patterns, url

USERNAME_LEN_MIN = 10 ** 10
USERNAME_LEN_MAX = 30 ** 10

def generate_random_username(attempts=10):
    qs = User.objects.all()
    r = random.SystemRandom()

    for i in xrange(attempts):
        username = str(r.randint(USERNAME_LEN_MIN, USERNAME_LEN_MAX))

        # ensure this alias is not already taken
        if not qs.filter(username=username).exists():
            return username

    raise StandardError('max attempts have been reached (n={0})'.format(attempts))

def validate_password(password, length):
    "Validates the given password."
    if len(password) < length:
        raise forms.ValidationError(_('The password provided must be at '
            'least {0} characters.'.format(length)))

    cnt = 0
    # contain lowercase letters?
    if re.search('[a-z]+', password):
        cnt += 1
    # contain uppercase letters?
    if re.search('[A-Z]+', password):
        cnt += 1
    # contain numbers?
    if re.search('[0-9]+', password):
        cnt += 1
    # contain symbols?
    if re.search('[^a-zA-Z0-9]+', password):
        cnt += 1

    if cnt < 3:
        raise forms.ValidationError(_('This password does not meet the minimum '
            'requirements.'))


def create_urls(backend):
    return patterns('',
        url(r'^activate/complete/$', 'django.views.generic.simple.direct_to_template', {
            'template': 'registration/activation_complete.html'
        }, name='registration_activation_complete'),
    
        # Activation keys get matched by \w+ instead of the more specific
        # [a-fA-F0-9]{40} because a bad activation key should still get to the view;
        # that way it can return a sensible "invalid key" message instead of a
        # confusing 404.
        url(r'^activate/(?P<activation_key>\w+)/$', 'registration.views.activate', {
            'backend': backend
        }, name='registration_activate'),

        url(r'^register/$', 'registration.views.register', {
            'backend': backend
        }, name='register'),
    
        url(r'^register/complete/$', 'django.views.generic.simple.direct_to_template', {
            'template': 'registration/registration_complete.html'
        }, name='registration-complete'),
        
        url(r'^register/closed/$', 'django.views.generic.simple.direct_to_template', {
            'template': 'registration/registration_closed.html'
        }, name='registration-disabled'),
    )



from django import forms
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import authenticate, forms as auth_forms
from registration.utils import generate_random_username

class RegistrationForm(auth_forms.UserCreationForm):
    """Extends the standard Django user creation form that supplies a few more
    robust defaults.

    .. note::

        To make it easier to manage registration vs. account management, this
        form should not be used for existing users to manage their accounts as
        performs "new user" operations that may override existing user
        information.

    """

    first_name = forms.CharField(label=_('First Name'), widget=forms.TextInput)
    last_name = forms.CharField(label=_('Last Name'), widget=forms.TextInput)
    email = forms.EmailField(widget=forms.TextInput(attrs={'type': 'email'}))

    class Meta(object):
        model = User
        fields = ('first_name', 'last_name', 'username', 'email')

    def clean_email(self):
        email = self.cleaned_data.get('email')
        try:
            User.objects.get(email__iexact=email)
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(_('An account is already registered with this email address.'))


class EmailOnlyRegistrationForm(RegistrationForm):
    # override the UserCreationForm field
    username = forms.CharField(required=False)

    class Meta(object):
        model = User
        fields = ('first_name', 'last_name', 'email')

    def save(self, commit=True):
        user = super(EmailOnlyRegistrationForm, self).save(commit=False)
        user.username = generate_random_username()
        if commit:
            user.save()
        return user


class EmailAuthenticationForm(auth_forms.AuthenticationForm):
    email = forms.CharField(label=_('Email'))
    password = forms.CharField(label=_('Password'), widget=forms.PasswordInput)
    username = forms.CharField(required=False)

    def clean(self):
        email = self.cleaned_data.get('email')
        password = self.cleaned_data.get('password')

        if email and password:
            # this is not a mistake.. Django assumes username will always be
            # used, but the auth backend accepts an email address
            self.user_cache = authenticate(username=email, password=password)
            if self.user_cache is None:
                raise forms.ValidationError(_('Please enter a correct email '
                    'and password. Note that both fields are case-sensitive.'))
            elif not self.user_cache.is_active:
                raise forms.ValidationError(_('This account is not active.'))
        self.check_for_test_cookie()
        return self.cleaned_data


class ModerationForm(forms.Form):
    status = forms.CharField()
    comment = forms.CharField(widget=forms.Textarea, required=False)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

# Python 2.7 has an importlib with import_module; for older Pythons,
# Django's bundled copy provides it.
try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module

DEFAULT_BACKEND_ALIAS = 'default'

REGISTRATION_BACKENDS = getattr(settings, 'REGISTRATION_BACKENDS', {
    DEFAULT_BACKEND_ALIAS: 'registration.backends.default.Backend',
})

def get_backend(alias):
    """Return an instance of a registration backend, given the dotted
    Python import path (as a string) to the backend class.

    If the backend cannot be located (e.g., because no such module
    exists, or because the module does not contain a class of the
    appropriate name), ``django.core.exceptions.ImproperlyConfigured``
    is raised.
    """
    if not REGISTRATION_BACKENDS.has_key(alias):
        raise ImproperlyConfigured('No registration backend named "{0}"'.format(alias))

    path = REGISTRATION_BACKENDS[alias]

    i = path.rfind('.')
    module, attr = path[:i], path[i+1:]

    try:
        mod = import_module(module)
    except ImportError, e:
        raise ImproperlyConfigured('Error loading registration backend {0}: "{1}"'.format(module, e))

    try:
        backend_class = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured('Module "{0}" does not define a registration backend named "{1}"'.format(module, attr))

    return backend_class()



from django.shortcuts import redirect
from django.shortcuts import render
from django.views.decorators.cache import never_cache
from django.contrib.auth.decorators import permission_required

from registration.backends import get_backend

def register(request, backend='default', template_name='registration/registration_form.html'):
    backend = get_backend(backend)

    # determine is registration is currently allowed. the ``request`` object
    # is passed which can be used to selectively disallow registration based on
    # the user-agent
    if not backend.registration_allowed(request):
        return redirect(*backend.registration_closed_redirect(request))

    form_class = backend.get_registration_form_class(request)

    if request.method == 'POST':
        form = form_class(request.POST, request.FILES)

        if form.is_valid():
            user = backend.register(request, form)
            return redirect(backend.post_registration_redirect(request, user))
    else:
        form = form_class()

    return render(request, template_name, {'form': form})

@never_cache
def verify(request, backend='default', template_name='registration/registration_verify.html', **kwargs):
    backend = get_backend(backend)
    profile = backend.get_profile(request, **kwargs)

    if profile:
        # check to see if moderation for this profile is required and whether or
        # not it is a verified account.
        if backend.moderation_required(request, profile):
            moderation_required = True
            backend.verify(request, profile, **kwargs)
        else:
            moderation_required = False
            # attempt to activate this user
            backend.activate(request, profile, **kwargs)

    return render(request, template_name, {
        'profile': profile,
        'moderation_required': moderation_required,
    })

@never_cache
def moderate(request, backend='default', template_name='registration/registration_moderate.html', **kwargs):
    backend = get_backend(backend)
    profile = backend.get_profile(request, **kwargs)
    form_class = backend.get_moderation_form_class(request)

    if request.method == 'POST':
        form = form_class(request.POST)

        if form.is_valid():
            backend.moderate(request, form, profile, **kwargs)
            return redirect(backend.post_moderation_redirect(request, profile))
    else:
        form = form_class()

    return render(request, template_name, {
        'form': form,
        'profile': profile,
    })

@permission_required('registration.change_registrationprofile')
def moderate_list(request, backend='default', template_name='registration/registration_moderate_list.html'):
    backend = get_backend(backend)
    profiles = backend.get_unmoderated_profiles(request)

    return render(request, template_name, {
        'profiles': profiles,
    })



from datetime import datetime, timedelta

from django.db import models, transaction
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from registration.managers import RegistrationManager

class RegistrationProfile(models.Model):
    """A simple profile which stores an activation key for use during user
    account registration.

    Generally, you will not want to interact directly with instances of this
    model; the provided manager includes methods for creating and activating
    new accounts, as well as for cleaning out accounts which have never been
    activated.

    While it is possible to use this model as the value of the
    ``AUTH_PROFILE_MODULE`` setting, it's not recommended that you do so. This
    model's sole purpose is to store data temporarily during account
    registration and activation.
    """

    user = models.OneToOneField(User, related_name='registration_profile',
        verbose_name=_('user'))

    activation_key = models.CharField(_('activation key'), max_length=40)

    # for moderated account registration, this denotes the user has verified
    # they account e.g. clicked on a link sent to their email address
    verified = models.BooleanField(_('verified'), default=False)

    # denotes this user has been activated from a registration's perspective.
    # note, this is independent of ``User.is_active``
    activated = models.BooleanField(_('activated'), default=False)

    # denotes the user has been moderated
    moderated = models.BooleanField(_('moderated'), default=False)

    # denotes the user has been moderated
    moderator = models.ForeignKey(User, related_name='moderated_profiles',
        null=True, verbose_name=_('moderator'))

    # the time the user was moderated
    moderation_time = models.DateTimeField(_('moderation_time'), null=True)

    objects = RegistrationManager()

    class Meta(object):
        verbose_name = _('registration profile')
        verbose_name_plural = _('registration profiles')

    def __unicode__(self):
        return u'Registration Profile for %s' % self.user

    def save(self, *args, **kwargs):
        if not self.moderation_time and self.moderated:
            self.moderation_time = datetime.now()
        super(RegistrationProfile, self).save(*args, **kwargs)

    @transaction.commit_on_success
    def activate(self):
        user = self.user
        user.is_active = True
        user.save()
        self.activated = True
        self.save()
        return user

    def activation_expired(self, activation_days=None):
        """Determine whether this ``RegistrationProfile``'s activation key has
        expired, returning a boolean -- ``True`` if the key has expired.

        Key expiration is determined by a two-step process:

        1. If the user has already activated, the flag ``activated`` will be
        set to true

        2. Otherwise, the date the user signed up is incremented by the number
        of days specified in the setting ``ACCOUNT_ACTIVATION_DAYS`` (which
        should be the number of days after signup during which a user is
        allowed to activate their account); if the result is less than or equal
        to the current date, the key has expired and this method returns
        ``True``.
        """
        # if this is not set or is 0, always return False (no expiration)
        if not activation_days:
            return False

        # check is this has expired given the activation days
        expiration_date = timedelta(days=activation_days)
        return self.user.date_joined + expiration_date <= datetime.now()

    # for the admin..
    activation_expired.boolean = True



from django.conf.urls import patterns, url

urlpatterns = patterns('django.contrib.auth.views', 
    url(r'^login/$', 'login', {
        'template_name': 'registration/login.html'
    }, name='login'),
    
    url(r'^logout/$', 'logout_then_login', {
        'template_name': 'registration/logout.html'
    }, name='logout'),

    url(r'^password/reset/$', 'password_reset',
        name='password-reset'),

    url(r'^password/reset/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        'password_reset_confirm', name='password-reset-confirm'),

    url(r'^password/reset/complete/$', 'password_reset_complete',
        name='password-reset-complete'),
)



import sys
from setuptools import setup, find_packages

packages = find_packages(exclude=['tests', '*.tests', '*.tests.*', 'tests.*'])

kwargs = {
    'packages': packages,
    'include_package_data': True,
    'install_requires': [
        'django>=1.4',
    ],

    'name': 'django-registration2',
    'version': __import__('registration').get_version(),
    'author': 'Byron Ruth',
    'author_email': 'b@devel.io',
    'description': 'Registration and moderation utilites',
    'license': 'BSD',
    'keywords': 'registration moderation',
    'url': 'https://github.com/bruth/django-registration2',
    'classifiers': [
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
}

setup(**kwargs)



from django.conf import settings
from django.db import transaction
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from django.contrib.sites.models import Site, RequestSite

from registration import signals
from registration.forms import RegistrationForm, ModerationForm
from registration.models import RegistrationProfile

def send_registration_email(backend, request, profile, **kwargs):
    # get the current site
    if Site._meta.installed:
        site = Site.objects.get_current()
    else:
        site = RequestSite(request)

    # send a new user registration email explaining the next steps
    subject = render_to_string('registration/registration_subject.txt', {
        'site': site
    })
    # no newlines
    subject = ''.join(subject.splitlines())

    message = render_to_string('registration/registration_email.txt', {
        'site': site,
        'profile': profile,
        'moderated': backend.moderation_required(request, profile),
        'expiration_days': backend.get_activation_days(request),
    })

    profile.user.email_user(subject, message, settings.DEFAULT_FROM_EMAIL)

def send_moderator_email(backend, request, profile):
    # get the current site
    if Site._meta.installed:
        site = Site.objects.get_current()
    else:
        site = RequestSite(request)

    # send a new user registration email explaining the next steps
    subject = render_to_string('registration/moderator_subject.txt', {
        'site': site
    })

    # no newlines
    subject = ''.join(subject.splitlines())

    message = render_to_string('registration/moderator_email.txt', {
        'site': site,
        'profile': profile,
        'expiration_days': backend.get_activation_days(request),
    })

    # send an email to all account moderators
    moderators = (x[1] for x in backend.get_moderators(request))
    send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, moderators)

def send_acceptance_email(request, profile, **context):
    # get the current site
    if Site._meta.installed:
        site = Site.objects.get_current()
    else:
        site = RequestSite(request)

    # send a new user registration email explaining the next steps
    subject = render_to_string('registration/acceptance_subject.txt', {
        'site': site
    })

    # no newlines
    subject = ''.join(subject.splitlines())

    context.update({
        'site': site,
        'profile': profile,
    })

    message = render_to_string('registration/acceptance_email.txt', context)

    profile.user.email_user(subject, message, settings.DEFAULT_FROM_EMAIL)


class Backend(object):
    def _send_registration_email(self, *args, **kwargs):
        send_registration_email(self, *args, **kwargs)

    def _send_moderator_email(self, *args, **kwargs):
        send_moderator_email(self, *args, **kwargs)

    def _send_acceptance_email(self, *args, **kwargs):
        send_acceptance_email(*args, **kwargs)

    def get_profile(self, request, activation_key):
        "Returns a single profile by ``activation_key``."
        try:
            return self.get_profiles(request).get(activation_key=activation_key)
        except RegistrationProfile.DoesNotExist:
            pass

    def get_profiles(self, request, **kwargs):
        "Returns a QuerySet of registration profiles."
        return RegistrationProfile.objects.select_related('user').filter(**kwargs)

    def get_unverified_profiles(self, request):
        "Returns unverified profiles."
        return self.get_profiles(request, verified=False)

    def get_unmoderated_profiles(self, request):
        "Returns verified, unmoderated profiles."
        return self.get_profiles(request, verified=True, moderated=False)

    def get_inactivated_profiles(self, request):
        "Returns verified, non-activated profiles."
        return self.get_profiles(request, verified=True, activated=False)

    @transaction.commit_on_success
    def register(self, request, form, **kwargs):
        "Post-form validation registration logic."
        if Site._meta.installed:
            site = Site.objects.get_current()
        else:
            site = RequestSite(request)

        user = form.save(commit=False)
        user.is_active = False
        user.save()

        profile = RegistrationProfile.objects.create_profile(user)

        self._send_registration_email(request, profile)
        signals.user_registered.send(sender=self.__class__, user=user,
            request=request, backend=self)

        return user

    @transaction.commit_on_success
    def verify(self, request, profile, **kwargs):
        """Given an activation key, mark the account as being verified for
        moderation. Send an email to all account moderators on the first
        occurence.

        The status of the verification is returned which denotes whether the
        verification was successful or the account has already been verified.
        """
        if not profile.verified:
            profile.verified = True
            profile.save()

            signals.user_verified.send(sender=self.__class__, user=profile.user,
                request=request, backend=self)

            # if moderation is required, email moderators, otherwise activate
            # the user's profile immediately
            if self.moderation_required(request, profile):
                self._send_moderator_email(request, profile)
            else:
                self.activate(request, profile, **kwargs)

    def activate(self, request, profile, **kwargs):
        """Given an an activation key, look up and activate the user account
        corresponding to that key (if possible).

        After successful activation, the signal ``user_activated`` will be
        sent, with the newly activated ``User`` as the keyword argument
        ``user`` and the class of this backend as the sender.
        """
        if not profile.activated and not profile.activation_expired():
            profile.activate()
            signals.user_activated.send(sender=self.__class__, user=profile.user,
                request=request, backend=self)

    @transaction.commit_on_success
    def moderate(self, request, form, profile, **kwargs):
        if not profile.moderated:
            profile.moderated = True
            profile.moderator = request.user
            profile.save()

            # XXX ghetto and fragile..
            if form.cleaned_data['status'].lower() == 'approve':
                self.activate(request, profile, **kwargs)

            self._send_acceptance_email(request, profile, **form.cleaned_data)

    def registration_allowed(self, request):
        """
        Indicate whether account registration is currently permitted,
        based on the value of the setting ``REGISTRATION_OPEN``. This
        is determined as follows:

        * If ``REGISTRATION_OPEN`` is not specified in settings, or is
          set to ``True``, registration is permitted.

        * If ``REGISTRATION_OPEN`` is both specified and set to
          ``False``, registration is not permitted.

        """
        return getattr(settings, 'REGISTRATION_OPEN', True)

    def moderation_required(self, request, profile=None):
        "Indicate whether account moderation is enabled."
        return getattr(settings, 'REGISTRATION_MODERATION', False)

    def get_moderators(self, request):
        "Returns a tuple of moderators."
        return getattr(settings, 'REGISTRATION_MODERATORS', settings.MANAGERS)

    def get_activation_days(self, request):
        "Returns the number of days allowed for activation"
        return getattr(self, 'REGISTRATION_ACTIVATION_DAYS', 0)

    def get_registration_form_class(self, request):
        "Return the form class used for user registration."
        return RegistrationForm

    def get_moderation_form_class(self, request):
        "Return the form class used for user moderation."
        return ModerationForm

    def post_registration_redirect(self, request, user):
        "Return the ``reverse`` arguments for post-registration."
        return reverse('registration-complete')

    def post_moderation_redirect(self, request, user):
        "Return the ``reverse` arguments for post-moderation."
        return reverse('moderate-registration-list')




from django.conf.urls import url, patterns, include
from django.views.generic.base import TemplateView
from registration.views import verify, register

urlpatterns = patterns('',
    url(r'^register/$', register, name='register'),

    url(r'^register/complete/$', TemplateView.as_view(
        template_name='registration/registration_complete.html'
    ), name='registration-complete'),

    # Activation keys get matched by \w+ instead of the more specific
    # [a-fA-F0-9]{40} because a bad activation key should still get to the view;
    # that way it can return a sensible "invalid key" message instead of a
    # confusing 404.
    url(r'^verify/(?P<activation_key>\w+)/$', verify,
        name='verify-registration'),

    url(r'^register/closed/$', TemplateView.as_view(
        template_name='registration/registration_closed.html'
    ), name='registration-disabled'),

    url(r'^moderate/(?P<activation_key>\w+)/$', 'registration.views.moderate',
        name='moderate-registration'),

    url(r'^moderate/$', 'registration.views.moderate_list',
        name='moderate-registration-list'),

    url(r'', include('registration.auth_urls')),
)
